"""
rest channel implementation
"""
import inspect
import re
from dataclasses import is_dataclass

from typing import get_type_hints, TypeVar, Annotated, Callable, get_origin, get_args, Type
from pydantic import BaseModel

from aspyx.reflection import DynamicProxy, Decorators

from .channels import HTTPXChannel
from .service import channel, ServiceCommunicationException

T = TypeVar("T")

class BodyMarker:
    pass

Body = lambda t: Annotated[t, BodyMarker]

class QueryParamMarker:
    pass

QueryParam = lambda t: Annotated[t, QueryParamMarker]

# decorators

def rest(url=""):
    """
    mark service interfaces to add a url prefix

    Args:
        url: prefix that will be added to all urls
    """
    def decorator(cls):
        Decorators.add(cls, rest, url)

        return cls
    return decorator

def get(url: str):
    """
    methods marked with `get` will be executed by calling a http get request.

    Args:
        url: the url
    """
    def decorator(cls):
        Decorators.add(cls, get, url)

        return cls
    return decorator


def post(url: str):
    """
    methods marked with `post` will be executed by calling a http get request.
    The body parameter should be marked with `Body(<param>)`

    Args:
        url: the url
    """
    def decorator(cls):
        Decorators.add(cls, post, url)

        return cls

    return decorator

def put(url: str):
    """
    methods marked with `put` will be executed by calling a http put request.

    Args:
        url: the url
    """
    def decorator(cls):
        Decorators.add(cls, put, url)

        return cls

    return decorator

def delete(url: str):
    """
    methods marked with `delete` will be executed by calling a http delete request.

    Args:
        url: the url
    """
    def decorator(cls):
        Decorators.add(cls, delete, url)

        return cls

    return decorator

def patch(url: str):
    """
    methods marked with `patch` will be executed by calling a http patch request.

    Args:
        url: the url
    """
    def decorator(cls):
        Decorators.add(cls, patch, url)

        return cls

    return decorator

@channel("rest")
class RestChannel(HTTPXChannel):
    """
    A rest channel executes http requests as specified by the corresponding decorators and annotations,
    """
    __slots__ = [
        "signature",
        "url_template",
        "type",
        "calls",
        "return_type",
        "path_param_names",
        "query_param_names",
        "body_param_name"
    ]

    # local class

    class Call:
        def __init__(self, type: Type, method : Callable):
            self.signature = inspect.signature(method)

            param_names = list(self.signature.parameters.keys())
            param_names.remove("self")

            prefix = ""
            if Decorators.has_decorator(type, rest):
                prefix = Decorators.get_decorator(type, rest).args[0]

            # find decorator

            self.type = "get"
            self.url_template = ""

            decorators = Decorators.get_all(method)

            for decorator in [get, post, put, delete, patch]:
                descriptor = next((descriptor for descriptor in decorators if descriptor.decorator is decorator), None)
                if descriptor is not None:
                    self.type = decorator.__name__
                    self.url_template = prefix + descriptor.args[0]

            # parameters

            self.path_param_names = set(re.findall(r"{(.*?)}", self.url_template))

            for param_name in self.path_param_names:
                param_names.remove(param_name)

            hints = get_type_hints(method, include_extras=True)

            self.body_param_name = None
            self.query_param_names = set()

            for param_name, hint in hints.items():
                if get_origin(hint) is Annotated:
                    metadata = get_args(hint)[1:]

                    if BodyMarker in metadata:
                        self.body_param_name = param_name
                        param_names.remove(param_name)
                    elif QueryParamMarker in metadata:
                        self.query_param_names.add(param_name)
                        param_names.remove(param_name)

            # check if something is missing

            if param_names:
                # check body params
                if self.type in ("post", "put", "patch"):
                    if self.body_param_name is None:
                        candidates = [
                            (name, hint)
                            for name, hint in hints.items()
                            if name not in self.path_param_names
                        ]
                        # find first dataclass or pydantic argument
                        for name, hint in candidates:
                            typ = hint
                            if get_origin(typ) is Annotated:
                                typ = get_args(typ)[0]
                            if (
                                    (isinstance(typ, type) and issubclass(typ, BaseModel))
                                    or is_dataclass(typ)
                            ):
                                self.body_param_name = name
                                param_names.remove(name)
                                break

            # the rest are query params

            for param in param_names:
                self.query_param_names.add(param)

            # return type

            self.return_type = get_type_hints(method)['return']

    # constructor

    def __init__(self):
        super().__init__()

        self.calls : dict[Callable, RestChannel.Call] = {}

    # internal

    def get_call(self, type: Type ,method: Callable):
        call = self.calls.get(method, None)
        if call is None:
            call = RestChannel.Call(type, method)
            self.calls[method] = call

        return call

    # override

    async def invoke_async(self, invocation: 'DynamicProxy.Invocation'):
        call = self.get_call(invocation.type, invocation.method)

        bound = call.signature.bind(self, *invocation.args, **invocation.kwargs)
        bound.apply_defaults()
        arguments = bound.arguments

        # url

        url = call.url_template.format(**arguments)
        query_params = {k: arguments[k] for k in call.query_param_names if k in arguments}
        body = {}
        if call.body_param_name is not None:
            body = self.to_dict(arguments.get(call.body_param_name))

        # call

        try:
            result = None
            if call.type in ["get", "put", "delete"]:
                result = await self.request_async(call.type, self.get_url() + url, params=query_params, timeout=self.timeout)

            elif call.type == "post":
                result = await self.request_async("post", self.get_url() + url, params=query_params, json=body, timeout=self.timeout)

            return self.get_deserializer(invocation.type, invocation.method)(result.json())
        except ServiceCommunicationException:
            raise

        except Exception as e:
            raise ServiceCommunicationException(f"communication exception {e}") from e

    def invoke(self, invocation: DynamicProxy.Invocation):
        call = self.get_call(invocation.type, invocation.method)

        bound = call.signature.bind(self,*invocation.args, **invocation.kwargs)
        bound.apply_defaults()
        arguments = bound.arguments

        # url

        url = call.url_template.format(**arguments)
        query_params = {k: arguments[k] for k in call.query_param_names if k in arguments}
        body = {}
        if call.body_param_name is not None:
            body = self.to_dict(arguments.get(call.body_param_name))

        # call

        try:
            result = None
            if call.type in ["get", "put", "delete"]:
                result = self.request("get", self.get_url() + url, params=query_params, timeout=self.timeout)

            elif call.type == "post":
                result = self.request( "post", self.get_url() + url, params=query_params, json=body, timeout=self.timeout)

            return self.get_deserializer(invocation.type, invocation.method)(result.json())
        except ServiceCommunicationException:
            raise

        except Exception as e:
            raise ServiceCommunicationException(f"communication exception {e}") from e



"""
authorization logic
"""
import inspect
from abc import abstractmethod, ABC
from typing import Optional, Callable

from aspyx.di import injectable, inject, order
from aspyx.di.aop import Invocation
from aspyx.reflection import TypeDescriptor, Decorators

def get_method_class(method):
    if inspect.ismethod(method) or inspect.isfunction(method):
        qualname = method.__qualname__
        parts = qualname.split('.')
        if len(parts) > 1:
            cls_name = parts[-2]
            module = inspect.getmodule(method)
            if module:
                for name, obj in inspect.getmembers(module, inspect.isclass):
                    if name == cls_name and hasattr(obj, method.__name__):
                        return obj
    return None

@injectable()
class AuthorizationManager:
    """
    The authorization manager is used to remember and execute pluggable authorization checks.
    """
    class Authorization():
        """
        Base class for authorization checks
        """
        def authorize(self, invocation: Invocation):
            """
            execute the authorization check. Throws an exception in case of violations
            """

    class AuthorizationFactory(ABC):
        """
        An authorization factory is used to create possible authorization checks given a method descriptor
        """

        def __init__(self, order = 0):
            self.order = order

        @abstractmethod
        def compute_authorization(self, method_descriptor: TypeDescriptor.MethodDescriptor) -> Optional['AuthorizationManager.Authorization']:
            """
            return a possible authorization check given a method descriptor
            Args:
                method_descriptor: the corresponding method descriptor

            Returns:
                an authorization check or None
            """

    # constructor

    def __init__(self):
        self.factories : list[AuthorizationManager.AuthorizationFactory] = []
        self.checks : dict[Callable, list[AuthorizationManager.Authorization]] = {}

    # public

    def register_factory(self, factory: 'AuthorizationManager.AuthorizationFactory'):
        self.factories.append(factory)

        self.factories.sort(key=lambda factory: factory.order)

    # internal

    def compute_checks(self, func: Callable) -> list[Authorization]:
        checks = []

        clazz = get_method_class(func)

        descriptor = TypeDescriptor.for_type(clazz).get_method(func.__name__)

        for factory in self.factories:
            check = factory.compute_authorization(descriptor)
            if check is not None:
                checks.append(check)

        return checks

    def get_checks(self, func: Callable) -> list[Authorization]:
        """
        return a list of authorization checks given a function.

        Args:
            func: the corresponding function.

        Returns:
            list of authorization checks
        """
        checks = self.checks.get(func, None)
        if checks is None:
            checks = self.compute_checks(func)
            self.checks[func] = checks
            print(checks)

        return checks

    def authorize(self, invocation: Invocation):
        for check in self.get_checks(invocation.func):
            check.authorize(invocation)

class AbstractAuthorizationFactory(AuthorizationManager.AuthorizationFactory):
    """
    Abstract base class for authorization factories
    """

    # constructor

    def __init__(self):
        super().__init__(0)

        if Decorators.has_decorator(type(self), order):
            self.order = Decorators.get_decorator(type(self), order).args[0]

    # inject

    @inject()
    def set_authorization_manager(self, authorization_manager: AuthorizationManager):
        authorization_manager.register_factory(self)



"""
registries for components in aspyx service
"""
import re
import threading
from abc import abstractmethod
import time
from typing import Optional

import consul

from aspyx.di.configuration import inject_value
from aspyx.util import StringBuilder
from aspyx.di import on_init
from .healthcheck import HealthCheckManager, HealthStatus

from .server import Server
from .service import ComponentRegistry, Channel, ChannelInstances, ServiceManager, ComponentDescriptor, Component, ChannelAddress

class ConsulComponentRegistry(ComponentRegistry):
    """
    A specialized registry using consul.
    A polling mechanism is used to identify changes in the component health.
    """
    # constructor

    def __init__(self, port: int, consul: consul.Consul):
        self.port = port
        self.ip = Server.get_local_ip()
        self.running = False
        self.consul = consul
        self.watchdog = None
        self.interval = 5
        self.last_index = {}
        self.component_addresses : dict[str, dict[str, ChannelInstances]] = {} # comp -> channel -> address
        self.watch_channels : list[Channel] = []
        self.watchdog_interval = 5
        self.healthcheck_interval = "10s"
        self.healthcheck_timeout= "5s"
        self.healthcheck_deregister = "5m"

    # injections

    @inject_value("consul.watchdog.interval", default=5)
    def set_watchdog_interval(self, interval):
        self.watchdog_interval = interval

    @inject_value("consul.healthcheck.interval", default="10s")
    def set_healthcheck_interval(self, interval):
        self.healthcheck_interval = interval

    @inject_value("consul.healthcheck.timeout", default="3s")
    def set_healthcheck_timeout(self, interval):
        self.healthcheck_timeout = interval

    @inject_value("consul.healthcheck.deregister", default="5m")
    def set_healthcheck_deregister(self, interval):
        self.healthcheck_deregister = interval

    # lifecycle hooks

    @on_init()
    def setup(self):
        # create consul client

        self.running = True

        # start thread

        self.watchdog = threading.Thread(target=self.watch_consul, daemon=True)
        self.watchdog.start()

    def inform_channels(self, old_address: ChannelInstances, new_address: Optional[ChannelInstances]):
        for channel in self.watch_channels:
            if channel.address is old_address:
                channel.set_address(new_address)


    def watch_consul(self):
        while self.running:
            # check services

            for component, old_addresses in self.component_addresses.items():
                old_addresses = dict(old_addresses) # we will modify it...
                new_addresses = self.fetch_addresses(component, wait="1s")

                # compare

                changed = False
                for channel_name, address in new_addresses.items():
                    service_address = old_addresses.get(channel_name, None)
                    if service_address is None:
                        ServiceManager.logger.info("new %s address for %s", channel_name, component)
                        changed = True
                    else:
                        if service_address != address:
                            changed = True

                            ServiceManager.logger.info("%s address for %s changed", channel_name, component)

                            # inform channels

                            self.inform_channels(service_address, address)

                        # delete

                        del old_addresses[channel_name]

                # watchout for deleted addresses

                for channel_name, address in old_addresses.items():
                    ServiceManager.logger.info("deleted %s address for %s", channel_name, component)

                    changed = True

                    # inform channel

                    self.inform_channels(address, None)

                # replace ( does that work while iterating )

                if changed:
                    self.component_addresses[component] = new_addresses

            # time to sleep

            time.sleep(self.watchdog_interval)

    @abstractmethod
    def watch(self, channel: Channel) -> None:
        self.watch_channels.append(channel)

   # public

    def register_service(self, name, service_id, health: str, tags=None, meta=None) -> None:
        self.consul.agent.service.register(
            name=name,
            service_id=service_id,
            address=self.ip,
            port=self.port,
            tags=tags or [],
            meta=meta or {},
            check=consul.Check().http(
                url=f"http://{self.ip}:{self.port}{health}",
                interval=self.healthcheck_interval,
                timeout=self.healthcheck_timeout,
                deregister=self.healthcheck_deregister)
            )

    def deregister(self, descriptor: ComponentDescriptor[Component]) -> None:
        name = descriptor.name

        service_id = f"{self.ip}:{self.port}:{name}"

        self.consul.agent.service.deregister(service_id)

    def stop(self):
        self.running = False
        if self.watchdog is not None:
            self.watchdog.join()
            self.watchdog = None

    # private

    def fetch_addresses(self, component: str, wait=None) -> dict[str, ChannelInstances]:
        addresses : dict[str, ChannelInstances] = {} # channel name -> ServiceAddress

        index, nodes = self.consul.health.service(component, passing=True, index=self.last_index.get(component, None), wait=wait)
        self.last_index[component] = index

        for node in nodes:
            service = node["Service"]

            meta = service.get('Meta')

            channels = meta.get("channels").split(",")

            for channel in channels:
                match = re.search(r"([\w-]+)\((.*)\)", channel)

                channel_name = match.group(1)
                url = match.group(2)

                address = addresses.get(channel, None)
                if address is None:
                    address = ChannelInstances(component=component, channel=channel_name)
                    addresses[channel] = address

                address.urls.append(url)

        # done

        return addresses

    # implement ComponentRegistry

    def register(self, descriptor: ComponentDescriptor[Component], addresses: list[ChannelAddress]):
        name = descriptor.name

        id = f"{self.ip}:{self.port}:{name}"

        builder = StringBuilder()
        first = True
        for address in addresses:
            if not first:
                builder.append(",")

            builder.append(address.channel).append("(").append(address.uri).append(")")

            first = False

        addresses = str(builder)

        self.register_service(name, id, descriptor.health, tags =["component"], meta={"channels": addresses})

    def get_addresses(self, descriptor: ComponentDescriptor) -> list[ChannelInstances]:
        component_addresses = self.component_addresses.get(descriptor.name, None)
        if component_addresses is None:
            component_addresses = self.fetch_addresses(descriptor.name)

            # only cache if non-empty

            if component_addresses:
                self.component_addresses[descriptor.name] = component_addresses

        return list(component_addresses.values())

    # 200–299	passing	Service is healthy (OK, Created, No Content…)
    # 429	warning	Rate limited or degraded
    # 300–399	warning	Redirects interpreted as potential issues
    # 400–499	critical	Client errors (Bad Request, Unauthorized…)
    # 500–599	critical	Server errors (Internal Error, Timeout…)
    # Other / No response	critical	Timeout, connection refused, etc.

    def map_health(self, health: HealthCheckManager.Health) -> int:
        if health.status is HealthStatus.OK:
            return 200
        elif health.status is HealthStatus.WARNING:
            return 429
        else:
            return 500



"""
session related module
"""
from abc import ABC, abstractmethod
import contextvars
from typing import Type, Optional, Callable, Any, TypeVar
from datetime import datetime, timezone, timedelta
from cachetools import TTLCache

from aspyx.di import injectable


class Session:
    """
    Base class for objects covers data related to a server side session.
    """
    def __init__(self):
        pass

T = TypeVar("T")

class SessionContext:
    # class properties

    # current_session = ThreadLocal[Session]()
    current_session = contextvars.ContextVar("session")

    @classmethod
    def get(cls, type: Type[T]) -> T:
        """
        return the current session associated with the context
        Args:
            type:  the session type

        Returns:
            the current session
        """
        return cls.current_session.get()

    @classmethod
    def set(cls, session: Session) -> None:
        """
        set the current session in the context
        Args:
            session: the session
        """
        cls.current_session.set(session)

    @classmethod
    def clear(cls) -> None:
        """
        delete the current session
        """
        cls.current_session.set(None)  # clear()

@injectable()
class SessionManager(SessionContext):
    """
    A SessionManager controls the lifecycle of sessions and is responsible to establish a session context local.
    """

    # local classes

    class Storage(ABC):
        @abstractmethod
        def store(self, token: str, session: Session, ttl_seconds: int):
            pass

        @abstractmethod
        def read(self, token: str) -> Optional[Session]:
            pass

    class InMemoryStorage(Storage):
        """
        InMemoryStorage is a simple in-memory storage for sessions.
        It uses a TTLCache to store sessions with a time-to-live.
        """
        # constructor

        def __init__(self, max_size = 1000, ttl = 3600):
            self.cache = TTLCache(maxsize=max_size, ttl=ttl)

        # implement

        def store(self, token: str, session: 'Session', ttl_seconds: int):
            expiry_time = datetime.now(timezone.utc) + timedelta(seconds=ttl_seconds)
            self.cache[token] = (session, expiry_time)

        def read(self, token: str) -> Optional['Session']:
            value = self.cache.get(token)
            if value is None:
                return None

            session, expiry = value
            if expiry < datetime.now(timezone.utc):
                del self.cache[token]
                return None

            return session

    # constructor

    def __init__(self, storage: 'SessionManager.Storage'):
        self.storage = storage
        self.session_factory : Optional[Callable[[Any], Session]] = None

    # public

    def set_factory(self, factory: Callable[..., Session]) -> None:
        """
        set a factory function that will be used to create a concrete session
        Args:
            factory: the function
        """
        self.session_factory = factory

    def create_session(self, *args, **kwargs) -> Session:
        """
        create a session given the arguments (usually a token, etc.)
        Args:
            args: rest args
            kwargs: keyword args

        Returns:
            the new session
        """
        return self.session_factory(*args, **kwargs)

    def store_session(self, token: str, session: Session, expiry: datetime):
        now = datetime.now(timezone.utc)
        ttl_seconds = max(int((expiry - now).total_seconds()), 0)

        self.storage.store(token, session, ttl_seconds)

    def read_session(self, token: str) -> Optional[Session]:
        return self.storage.read(token)



"""
Tests
"""
import pytest

from .common import TestAsyncService, TestAsyncRestService, Pydantic, Data, service_manager

pydantic = Pydantic(i=1, f=1.0, b=True, s="s")
data = Data(i=1, f=1.0, b=True, s="s")

@pytest.mark.asyncio(scope="function")
class TestAsyncRemoteService():
    async def test(self, service_manager):

        # dispatch json

        test_service = service_manager.get_service(TestAsyncService, preferred_channel="dispatch-json")

        result = await test_service.hello("hello")
        assert result == "hello"

        result_data = await test_service.data(data)
        assert result_data == data

        result_pydantic = await test_service.pydantic(pydantic)
        assert result_pydantic == pydantic

        # msgpack

        test_service = service_manager.get_service(TestAsyncService, preferred_channel="dispatch-msgpack")

        result = await test_service.hello("hello")
        assert result == "hello"

        result_data = await test_service.data(data)
        assert result_data == data

        result_pydantic = await test_service.pydantic(pydantic)
        assert result_pydantic == pydantic

        # rest

        test_service = service_manager.get_service(TestAsyncRestService, preferred_channel="rest")

        result = await test_service.get("hello")
        assert result == "hello"

        result = await test_service.put("hello")
        assert result =="hello"

        result = await test_service.delete("hello")
        assert result == "hello"

        result_pydantic = await test_service.post_pydantic("message", pydantic)
        assert result_pydantic == pydantic

        result_data = await test_service.post_data("message", data)
        assert result_data == data



"""
the server hosting the test services
"""
import logging
import os

from fastapi import FastAPI

from aspyx_service import FastAPIServer, RequestContext
from server import  ServerModule
from aspyx.util import Logger


Logger.configure(default_level=logging.DEBUG, levels={
    "httpx": logging.ERROR,
    "aspyx.di": logging.ERROR,
    "aspyx.di.aop": logging.ERROR,
    "aspyx.service": logging.ERROR
})

PORT = int(os.getenv("FAST_API_PORT", "8000"))

app = FastAPI()

app.add_middleware(RequestContext)
#app.add_middleware(TokenContextMiddleware)

ServerModule.fastapi = app

FastAPIServer.boot(ServerModule, host="0.0.0.0", port=PORT, start_thread= False)

if __name__ == "__main__":
    import uvicorn
    uvicorn.run("main:app", host="0.0.0.0", port=PORT, reload=True, log_level="warning", access_log=False)



"""
Tests
"""
import os
from typing import Optional

from consul import Consul
from fastapi import FastAPI

from aspyx_service import HealthCheckManager, ServiceModule, ConsulComponentRegistry, SessionManager, FastAPIServer

from client import ClientModule, TestService, TestAsyncService, Data, Pydantic, TestRestService, TestAsyncRestService, TestComponent

from aspyx_service.service import ChannelAddress, Server, \
    component_services, AbstractComponent, implementation, health, ComponentRegistry, ServiceManager
from aspyx.di import on_running, module, create
from aspyx.di.aop import Invocation, advice, error
from aspyx.exception import handle, ExceptionManager




# implementation classes

@implementation()
class TestServiceImpl(TestService):
    def __init__(self):
        pass

    def hello(self, message: str) -> str:
        return message

    def throw(self, message: str) -> str:
        raise Exception(message)

    def data(self, data: Data) -> Data:
        return data

    def pydantic(self, data: Pydantic) -> Pydantic:
        return data

@implementation()
class TestAsyncServiceImpl(TestAsyncService):
    def __init__(self):
        pass

    async def hello(self, message: str) -> str:
        return message

    async def data(self, data: Data) -> Data:
        return data

    async def pydantic(self, data: Pydantic) -> Pydantic:
        return data

@implementation()
class TestRestServiceImpl(TestRestService):
    def __init__(self):
        pass

    def get(self, message: str) -> str:
        return message

    def put(self, message: str) -> str:
        return message

    def post_pydantic(self, message: str, data: Pydantic) -> Pydantic:
        return data

    def post_data(self, message: str, data: Data) -> Data:
        return data

    def delete(self, message: str) -> str:
        return message

@implementation()
class TestAsyncRestServiceImpl(TestAsyncRestService):
    def __init__(self):
        pass

    async def get(self, message: str) -> str:
        return message

    async def put(self, message: str) -> str:
        return message

    async def post_pydantic(self, message: str, data: Pydantic) -> Pydantic:
        return data

    async def post_data(self, message: str, data: Data) -> Data:
        return data

    async def delete(self, message: str) -> str:
        return message

@implementation()
@health("/health")
@advice
class TestComponentImpl(AbstractComponent, TestComponent):
    # constructor

    def __init__(self):
        super().__init__()

        self.health_check_manager : Optional[HealthCheckManager] = None
        self.exception_manager = ExceptionManager()

    # exception handler

    @handle()
    def handle_exception(self, exception: Exception):
        print("caught exception!")
        return exception

    # aspects

    @error(component_services(TestComponent))
    def catch(self, invocation: Invocation):
        return self.exception_manager.handle(invocation.exception)

    # lifecycle

    @on_running()
    def setup_exception_handlers(self):
        self.exception_manager.collect_handlers(self)

    # implement

    async def get_health(self) -> HealthCheckManager.Health:
        return HealthCheckManager.Health()

    def get_addresses(self, port: int) -> list[ChannelAddress]:
        return [
            ChannelAddress("rest", f"http://{Server.get_local_ip()}:{port}"),
            ChannelAddress("dispatch-json", f"http://{Server.get_local_ip()}:{port}"),
            ChannelAddress("dispatch-msgpack", f"http://{Server.get_local_ip()}:{port}")
        ]

    def startup(self) -> None:
        print("### startup")

    def shutdown(self) -> None:
        print("### shutdown")

# module

@module(imports=[ClientModule, ServiceModule])
class ServerModule:
    fastapi: Optional[FastAPI] = None

    def __init__(self):
        pass

    #@create()
    #def create_yaml_source(self) -> YamlConfigurationSource:
    #    return YamlConfigurationSource(f"{Path(__file__).parent}/config.yaml")

    @create()
    def create_server(self, service_manager: ServiceManager, component_registry: ComponentRegistry) -> FastAPIServer:
        return FastAPIServer(self.fastapi, service_manager, component_registry)

    @create()
    def create_session_storage(self) -> SessionManager.Storage:
        return SessionManager.InMemoryStorage(max_size=1000, ttl=3600)

    @create()
    def create_registry(self) -> ComponentRegistry:
        return ConsulComponentRegistry(port=int(os.getenv("FAST_API_PORT", "8000")), consul=Consul(host="localhost", port=8500))
